"""update client

Revision ID: b8aa09688558
Revises: 0a075bcbeb8f
Create Date: 2019-07-23 16:33:44.773166

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'b8aa09688558'
down_revision = '0a075bcbeb8f'
branch_labels = None
depends_on = None

def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('client', sa.Column('_redirect_uris', sa.Text(), nullable=True))
    op.add_column('client', sa.Column('name', sa.String(length=40), nullable=True))
    # ### end Alembic commands ###

def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('client', 'name')
    op.drop_column('client', '_redirect_uris')
    # ### end Alembic commands ###
